package org.classpath.icedtea.pulseaudio;

import java.io.File;
import java.io.IOException;

import javax.sound.sampled.AudioFormat;
import javax.sound.sampled.AudioInputStream;
import javax.sound.sampled.AudioSystem;
import javax.sound.sampled.DataLine;
import javax.sound.sampled.Line;
import javax.sound.sampled.LineEvent;
import javax.sound.sampled.LineListener;
import javax.sound.sampled.LineUnavailableException;
import javax.sound.sampled.Mixer;
import javax.sound.sampled.SourceDataLine;
import javax.sound.sampled.TargetDataLine;
import javax.sound.sampled.UnsupportedAudioFileException;

import junit.framework.JUnit4TestAdapter;

import org.junit.After;
import org.junit.Assert;
import org.junit.Before;
import org.junit.Test;

public class PulseAudioTargetDataLineTest {

	private Mixer mixer;
	private TargetDataLine targetDataLine;

	AudioFormat aSupportedFormat = new AudioFormat(
			AudioFormat.Encoding.PCM_UNSIGNED, 44100f, 8, 1, 1, 10, true);

	public static junit.framework.Test suite() {
		return new JUnit4TestAdapter(PulseAudioTargetDataLineTest.class);
	}

	@Before
	public void setUp() throws LineUnavailableException {
		Mixer.Info[] mixerInfos = AudioSystem.getMixerInfo();
		Mixer.Info wantedMixerInfo = null;
		for (Mixer.Info mixerInfo : mixerInfos) {
			if (mixerInfo.getName().contains("PulseAudio")) {
				wantedMixerInfo = mixerInfo;
			}
		}
		Assert.assertNotNull(wantedMixerInfo);
		mixer = AudioSystem.getMixer(wantedMixerInfo);
		Assert.assertNotNull(mixer);
		mixer.open();
		targetDataLine = null;

	}

	@Test
	public void testOpenClose() throws LineUnavailableException {
		targetDataLine = (TargetDataLine) mixer.getLine(new Line.Info(
				TargetDataLine.class));
		Assert.assertNotNull(targetDataLine);
		targetDataLine.open();
		targetDataLine.close();
	}

	@Test
	public void testIsActiveAndIsOpen() throws LineUnavailableException {

		SourceDataLine line = (SourceDataLine) mixer.getLine(new DataLine.Info(
				SourceDataLine.class, aSupportedFormat, 1000));

		Assert.assertFalse(line.isActive());
		Assert.assertFalse(line.isOpen());
		line.open();
		Assert.assertTrue(line.isOpen());
		Assert.assertFalse(line.isActive());
		line.start();
		Assert.assertTrue(line.isOpen());
		Assert.assertTrue(line.isActive());
		line.stop();
		Assert.assertTrue(line.isOpen());
		Assert.assertFalse(line.isActive());
		line.close();
		Assert.assertFalse(line.isOpen());
		Assert.assertFalse(line.isActive());

	}

	@Test
	public void testOpenEvents() throws LineUnavailableException {
		targetDataLine = (TargetDataLine) mixer.getLine(new Line.Info(
				TargetDataLine.class));
		Assert.assertNotNull(targetDataLine);

		LineListener openListener = new LineListener() {
			private int calledCount = 0;

			@Override
			public void update(LineEvent event) {
				Assert.assertEquals(LineEvent.Type.OPEN, event.getType());
				calledCount++;
				Assert.assertEquals(1, calledCount);
			}

		};

		targetDataLine.addLineListener(openListener);
		targetDataLine.open();
		targetDataLine.removeLineListener(openListener);
		targetDataLine.close();

	}

	@Test
	public void testCloseEvents() throws LineUnavailableException {
		targetDataLine = (TargetDataLine) mixer.getLine(new Line.Info(
				TargetDataLine.class));
		Assert.assertNotNull(targetDataLine);

		LineListener closeListener = new LineListener() {
			private int calledCount = 0;

			@Override
			public void update(LineEvent event) {
				Assert.assertEquals(LineEvent.Type.CLOSE, event.getType());
				calledCount++;
				Assert.assertEquals(1, calledCount);
			}

		};

		targetDataLine.open();
		targetDataLine.addLineListener(closeListener);
		targetDataLine.close();
		targetDataLine.removeLineListener(closeListener);

	}

	int started = 0;
	int stopped = 0;

	@Test
	public void testStartedStopped() throws LineUnavailableException,
			UnsupportedAudioFileException, IOException {

		File soundFile = new File("testsounds/startup.wav");
		AudioInputStream audioInputStream = AudioSystem
				.getAudioInputStream(soundFile);
		AudioFormat audioFormat = audioInputStream.getFormat();

		TargetDataLine line;
		line = (TargetDataLine) mixer.getLine(new DataLine.Info(
				TargetDataLine.class, audioFormat));
		Assert.assertNotNull(line);

		started = 0;
		stopped = 0;

		line.open(audioFormat);

		LineListener startStopListener = new LineListener() {

			@Override
			public void update(LineEvent event) {
				if (event.getType() == LineEvent.Type.START) {
					started++;
					Assert.assertEquals(1, started);
				}

				if (event.getType() == LineEvent.Type.STOP) {
					System.out.println("Stopped event");
					stopped++;
					Assert.assertEquals(1, stopped);
				}
			}

		};

		line.addLineListener(startStopListener);

		line.start();

		line.drain();

		line.stop();
		line.close();

		Assert.assertEquals(1, started);
		Assert.assertEquals(1, stopped);

		started = 0;
		stopped = 0;

	}

	@Test
	public void testMixerKnowsAboutOpenLines() throws LineUnavailableException {
		targetDataLine = (TargetDataLine) mixer.getLine(new Line.Info(
				TargetDataLine.class));

		Assert.assertEquals(0, mixer.getTargetLines().length);
		targetDataLine.open();
		Assert.assertEquals(1, mixer.getTargetLines().length);
		targetDataLine.close();
		Assert.assertEquals(0, mixer.getTargetLines().length);

	}

	@Test
	public void testAllTargetLinesClosed() {
		Assert.assertEquals(0, mixer.getTargetLines().length);

	}

	@Test
	public void testTargetLineHasNoControls() throws LineUnavailableException {
		targetDataLine = (TargetDataLine) mixer.getLine(new Line.Info(
				TargetDataLine.class));

		targetDataLine.open();

		Assert.assertEquals(0, targetDataLine.getControls().length);

		targetDataLine.close();
	}

	@After
	public void tearDown() {
		mixer.close();
	}

}
